// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include <stdio.h>
#include <iostream.h>
#include <qpainter.h>
#include <qrect.h>
#include "PixData.h"
// Contains class definitions



// ***************
//  Here, we define the actual functions


// The overloaded converter function.  Converts a string of numbers to
// an int *.
//
void PixData::str2arr( char *buf, int *arr, int n )
{
  int i, eaten, nshift;

  nshift=0;
  for(i=0; i<n; i++)
	{
	sscanf((buf+nshift),"%d%n",&arr[i],&eaten);
	nshift += eaten+1;
	}
}



// The overloaded converter function.
// As above, except it converts to a float *.
//
void PixData::str2arr( char *buf, float *arr, int n )
{
  int i, eaten, nshift;

  nshift=0;
  for(i=0; i<n; i++)
	{
	sscanf((buf+nshift),"%f%n",&arr[i],&eaten);
	nshift += eaten+1;
	}
}



// Constructor for the class.
//
PixData::PixData(void)
{
   nobj = 0;
}



// Constructor for the class - overloaded version.
//
PixData::PixData( RawPixData rawdat, float bscale )
{
  int i,j;

  // First, perform the simple initializations.
  fixedScale = rawdat.fixed;
  nobj = rawdat.nobj;
  filled = rawdat.filled;
  drawtype = rawdat.drawtype;
  fg = black;
  bg = white;

  width0 = rawdat.w0;
  height0 = rawdat.h0;
  scale0 = rawdat.scale;
  dsize.setWidth( (int)(width0*scale0) );
  dsize.setHeight( (int)(height0*scale0) );

  // Now do the string <--> number conversions.
  npoints = new int[nobj];
  str2arr( rawdat.npointstr, npoints, nobj );
  
  dcoords0 = new float*[nobj];
  dcoords = new QCOORD*[nobj];
  for(i=0; i<nobj; i++)
	{
	// Allocate arrays.
	dcoords0[i] = new float[2*npoints[i]];
	dcoords[i] = new QCOORD[2*npoints[i]];

	// Read in the points.
	str2arr( rawdat.coordstr[i], dcoords0[i], 2*npoints[i] );

	// Now do the default scaling.
	for(j=0; j<2*npoints[i]; j++)
	   {
	   dcoords[i][j] = (QCOORD)(dcoords0[i][j]*scale0);
	   }
	}

  rescale( bscale );
}



// Copy Constructor for the class.
//
PixData::PixData(PixData& pd)
{
   this->operator=(pd);
}



// Overloaded operator.
//
PixData& PixData::operator=(PixData& pd)
{
   int i,j;

   nobj = pd.nobj;
   fixedScale = pd.fixedScale;
   filled = pd.filled;
   width0 = pd.width0;
   height0 = pd.height0;
   scale0 = pd.scale0;
   dsize.setWidth(pd.dsize.width());
   dsize.setHeight(pd.dsize.height());
   drawtype = pd.drawtype;

   npoints = new int[nobj];
   dcoords0 = new float*[nobj];
   dcoords = new QCOORD*[nobj];
   for(i=0; i<nobj; i++)
	 {
	 npoints[i] = pd.npoints[i];
	 dcoords0[i] = new float[2*npoints[i]];
	 dcoords[i] = new QCOORD[2*npoints[i]];
	 for(j=0; j<2*npoints[i]; j++)
	    {
	    dcoords[i][j] = pd.dcoords[i][j];
	    dcoords0[i][j] = pd.dcoords0[i][j];
	    }
	 }
   return (*this);
}



// Performs rescalings.
//
void PixData::rescale( float bscale )
{
  int i,j;
  float nucoord,scaleval;


  if( fixedScale )
	{
	return;
	}

  scaleval = scale0*bscale;
  dsize.setWidth( (int)(width0*scaleval) );
  dsize.setHeight( (int)(height0*scaleval) );

  for(i=0; i<nobj; i++)
	{
	for(j=0; j<2*npoints[i]; j++)
	   {
	   nucoord = dcoords0[i][j]*scaleval;
	   dcoords[i][j] = (QCOORD)nucoord;
	   }
	}
}



// Sets the fore- and background colors.  Call this *before*
// converting to a QPixmap!
//
void PixData::setColor( const QColor& fore, const QColor& back )
{
  fg = fore;
  bg = back;
}



// The converter.  You need to call this again if you rescale the
// data.
//
QPixmap PixData::toPixmap( void )
{
  int i;
  QPixmap *opx;
  QPainter pnt;
  QPointArray pts;
  QRect recpts;
  bool paint_ready;


  opx = new QPixmap( dsize );
  opx->fill( bg );

  paint_ready = pnt.begin( opx );
  if( !paint_ready )
	{
	cerr << "Fatal error in PixData::toPixmap()." << endl;
	cerr << "QPainter::begin() failed." << endl;
	cerr << "Aborting program." << endl;
	return (*opx);
	exit(-2);
	}
  pnt.setBackgroundColor( bg );
  pnt.setPen( fg );
  if(filled)
	{
	pnt.setBrush( fg );
	}
  
  for(i=0; i<nobj; i++)
	{
	if(drawtype[i] == 'c')
	   {
	   recpts.setCoords( dcoords[i][0], dcoords[i][1], 
					 dcoords[i][2], dcoords[i][3] );
	   pnt.drawEllipse( recpts );
	   }
	else
	   {
	   pts.setPoints( npoints[i], dcoords[i] );
	   if(drawtype[i] == 'p')
		 {
		 pnt.drawPolygon( pts );
		 }
	   else // default is 'l'==line segments
		 {
		 pnt.drawLineSegments( pts );
		 }
	   }
	}

  pnt.end();
  return (*opx);
}
